package net.minecraft.src;

import java.util.*;
import java.util.regex.*;

public class EntityDollAICutTree extends EntityAIBase
{
    private EntityDoll theDoll;
    private World theWorld;
    private float speed;
    private boolean isTarget, isCutting;
    private int targetX, cutX;
    private int targetY, cutY;
    private int targetZ, cutZ;
    private int counter;
    private int blockStrength;
    private int logID;

    public static String logBlock;
    public static String leavesBlock;
    public static int cutRange;
    public static double cutSpeed;
    
    public EntityDollAICutTree(EntityDoll entitydoll, float s)
    {
        theDoll = entitydoll;
        theWorld = entitydoll.worldObj;
        speed = s;
        counter = 0;
        setMutexBits(0x3);
    }

    @Override
    public boolean shouldExecute()
    {
        if(!theDoll.isEnable())
        {
            return false;
        }

        if(counter < 20)
        {
            ++counter;
            return false;
        }
        counter = 0;

        int dollposX = MathHelper.floor_double(theDoll.posX);
        int dollposY = MathHelper.floor_double(theDoll.posY + (double)theDoll.getEyeHeight());
        int dollposZ = MathHelper.floor_double(theDoll.posZ);
        int logCount = 0;
        int leavesCount = 0;
        Pattern logPattern = Pattern.compile(logBlock);
        Pattern leavesPattern = Pattern.compile(leavesBlock);
        Matcher logMatcher, leavesMatcher;
        targetX = cutRange;
        targetY = 0;
        targetZ = cutRange;
        isTarget = false;

        for(int dx=-cutRange; dx<=cutRange; ++dx)
        {
            for(int dz=-cutRange; dz<=cutRange; ++dz)
            {
                int dy = 0;
                int distanceToBlock = Math.abs(dx)+Math.abs(dz);
                if(distanceToBlock > cutRange)
                {
                    continue;
                }
                int bid = theWorld.getBlockId(dollposX+dx, dollposY+dy, dollposZ+dz);
                Block b = Block.blocksList[bid];
                if(b==null)
                {
                    continue;
                }
                String blockName = getBlockName(b);

                //؃ubNT
                logMatcher = logPattern.matcher(blockName);
                if(logMatcher.find())
                {
                    if(Math.abs(targetX)+Math.abs(targetZ) >distanceToBlock)
                    {
                        //؃ubN؂̈ꕔǂ
                        while(dollposY+dy > 0)
                        {
                            //̃ubN
                            int cbid = theWorld.getBlockId(dollposX+dx, dollposY+dy-1, dollposZ+dz);
                            if(cbid != bid)
                            {
                                break;
                            }
                            --dy;
                        }
                        int lcount = 0;
                        int bcount = 0;
                        while(dollposY+dy < theWorld.getHeight())
                        {
                            //̃ubN܂ŗאڂtubN̐𐔂
                            int cbid = theWorld.getBlockId(dollposX+dx, dollposY+dy, dollposZ+dz);
                            if(cbid != bid)
                            {
                                break;
                            }
                            
                            int[] nbid = new int[4];
                            nbid[0] = theWorld.getBlockId(dollposX+dx+1, dollposY+dy, dollposZ+dz);
                            nbid[1] = theWorld.getBlockId(dollposX+dx-1, dollposY+dy, dollposZ+dz);
                            nbid[2] = theWorld.getBlockId(dollposX+dx, dollposY+dy, dollposZ+dz+1);
                            nbid[3] = theWorld.getBlockId(dollposX+dx, dollposY+dy, dollposZ+dz-1);
                            for(int i=0; i<4; ++i)
                            {
                                Block lb = Block.blocksList[nbid[i]];
                                if(lb!=null)
                                {
                                    String lName = getBlockName(lb);
                                    leavesMatcher = leavesPattern.matcher(lName);
                                    
                                    if(leavesMatcher.find())
                                    {
                                        ++bcount;
                                    }
                                }
                            }
                            ++lcount;
                            ++dy;
                        }
                        //System.out.println(String.format("%d,%d",bcount,lcount));
                        if(bcount >= 4)
                        {
                            targetX = dx;
                            targetY = dy;
                            targetZ = dz;
                            isTarget = true;
                            logCount = bcount;
                            leavesCount = lcount;
                            logID = bid;
                        }
                    }
                }
            }
        }
        
        // o͕̍쐬
        StringBuffer msg = new StringBuffer(theDoll.getDollName() + " : ");

        if(!isTarget)
        {
            msg.append("No target");
            theDoll.chatMessage(msg.toString(),2);
            return false;
        }
        else
        {
            msg.append("Target[");
            msg.append(logCount);
            msg.append(",");
            msg.append(leavesCount);
            msg.append("]");
            theDoll.chatMessage(msg.toString(),1);
        }
        
        if(theDoll.isStandbyMode() || theDoll.isRideonMode())
        {
            return false;
        }

        targetX += dollposX;
        targetY += dollposY;
        targetZ += dollposZ;

        //̂ubŇ
        isCutting = false;
        for(int dx=-cutRange; dx<=cutRange; ++dx)
        {
            for(int dz=-cutRange; dz<=cutRange; ++dz)
            {
                if(Math.abs(dx)+Math.abs(dz) > cutRange)
                {
                    continue;
                }
                if(theWorld.getBlockId(targetX+dx, targetY+1, targetZ+dz) == logID)
                {
                    cutX = targetX+dx;
                    cutY = targetY+1;
                    cutZ = targetZ+dz;
                    isCutting = true;
                }
                if(isCutting)
                {
                    break;
                }
            }
            if(isCutting)
            {
                break;
            }
        }
        if(!isCutting)
        {
            cutX = targetX;
            cutY = targetY;
            cutZ = targetZ;
            isCutting = true;
        }
        
        Block b = Block.blocksList[theWorld.getBlockId(cutX, cutY, cutZ)];
        blockStrength = b == null ? 0 : MathHelper.floor_double(1.0 + 30.0*b.getBlockHardness(theWorld, cutX, cutY, cutZ)/cutSpeed);
        blockStrength = blockStrength < 0 ? 0 : blockStrength;

        theDoll.teleportToXYZ((double)targetX+0.5D, (double)targetY, (double)targetZ+0.5D, 0D);
        theDoll.getNavigator().clearPathEntity();
        
        return true;
    }

    @Override
    public boolean continueExecuting()
    {
        if(!theDoll.isEnable())
        {
            return false;
        }

        if(theDoll.isStandbyMode() || theDoll.isRideonMode())
        {
            return false;
        }

        return isCutting;
    }

    @Override
    public void startExecuting()
    {
        counter = 0;
    }

    @Override
    public void resetTask()
    {
    }

    @Override
    public void updateTask()
    {
        //̑Ώۂ
        theDoll.getLookHelper().setLookPosition(cutX+0.5F, cutY, cutZ+0.5F,
                                                20F, theDoll.getVerticalFaceSpeed());

        if(blockStrength <= 0)
        {
            //ubN
            Block b = Block.blocksList[theWorld.getBlockId(cutX, cutY, cutZ)];
            if(b != null)
            {
                StepSound stepsound = b.stepSound;
                b.dropBlockAsItem(theWorld, cutX, cutY, cutZ, theWorld.getBlockMetadata(cutX, cutY, cutZ), 0);
                theWorld.setBlockWithNotify(cutX, cutY, cutZ, 0);
                theWorld.playSoundEffect(cutX+0.5f, cutY+0.5f, cutZ+0.5f, stepsound.getBreakSound(),
                                         stepsound.getVolume(), stepsound.getPitch());
            }
            
            //̂ubŇ
            isCutting = false;
            for(int dx=-cutRange; dx<=cutRange; ++dx)
            {
                for(int dz=-cutRange; dz<=cutRange; ++dz)
                {
                    if(Math.abs(dx)+Math.abs(dz) > cutRange)
                    {
                        continue;
                    }
                    if(theWorld.getBlockId(targetX+dx, targetY, targetZ+dz) == logID)
                    {
                        cutX = targetX+dx;
                        cutY = targetY;
                        cutZ = targetZ+dz;
                        isCutting = true;
                    }
                    if(isCutting)
                    {
                        break;
                    }
                }
                if(isCutting)
                {
                    break;
                }
            }
            if(!isCutting)
            {
                --targetY;
                cutX = targetX;
                cutY = targetY;
                cutZ = targetZ;
                isCutting = true;
            }

            int bidNext = theWorld.getBlockId(cutX, cutY, cutZ);
            if(bidNext != logID)
            {
                isCutting = false;
            }
            else
            {
                Block bNext = Block.blocksList[bidNext];
                blockStrength = MathHelper.floor_double(1.0 + 30.0*bNext.getBlockHardness(theWorld, cutX, cutY, cutZ)/cutSpeed);
                blockStrength = blockStrength < 0 ? 0 : blockStrength;
            }
        }
        else if(blockStrength > 0 && blockStrength%4 == 0)
        {
            int bid = theWorld.getBlockId(cutX, cutY, cutZ);
            if(bid == 0)
            {
                isCutting = false;
                return;
            }
            Block b = Block.blocksList[bid];
            if(b != null)
            {
                StepSound stepsound = b.stepSound;
                theWorld.playSoundEffect(cutX+0.5f, cutY+0.5f, cutZ+0.5f, stepsound.getBreakSound(),
                                         (stepsound.getVolume() + 1.0f) / 8f, stepsound.getPitch() * 0.5f);
            }
        }
        --blockStrength;
    }

    private String getBlockName(Block b)
    {
        if(b == null)
        {
            return "";
        }
    
        String blockName = b.getBlockName();
        if(blockName == null)
        {
            blockName = String.format("Block%d", b.blockID);
        }
        else
        {
            int dot = blockName.indexOf(".");
            if(dot != -1)
            {
                blockName = blockName.substring(dot+1);
            }
        }
        return blockName;
    }
}

