package net.minecraft.src;

import java.util.*;
import java.util.regex.*;

public class EntityDollAIExplode extends EntityAIBase
{
    private EntityDoll theDoll;
    private World theWorld;
    private EntityLiving theTarget;
    private float speed;
    private int fuse;
    private int counter;
    
    public static String targetEntity;
    public static double searchRange;
    public static double searchHeight;
    public static float explodeStrength;
    
    public EntityDollAIExplode(EntityDoll entitydoll, float s)
    {
        theDoll = entitydoll;
        theWorld = entitydoll.worldObj;
        speed = s;
        counter = 0;
        setMutexBits(0x3);
    }

    @Override
    public boolean shouldExecute()
    {
        if(!theDoll.isEnable())
        {
            return false;
        }
        
        if(counter < 20)
        {
            ++counter;
            return false;
        }
        counter = 0;
        
        if(!theDoll.isPatrolMode())
        {
            return false;
        }
        List<EntityLiving> targetList = (List<EntityLiving>)(theWorld.getEntitiesWithinAABB(EntityLiving.class, theDoll.boundingBox.expand(searchRange, searchHeight, searchRange)));
        Pattern targetPattern = Pattern.compile(targetEntity);
        Matcher matcher;

        theTarget = null;
        for(EntityLiving e : targetList)
        {
            String name = e.getEntityString();
            if(name == null)
            {
                continue;
            }
            matcher = targetPattern.matcher(name);

            if(matcher.find())
            {
                if(theTarget == null)
                {
                    theTarget = e;
                }
                else
                {
                    if(theDoll.getDistanceSqToEntity(theTarget)
                       > theDoll.getDistanceSqToEntity(e))
                    {
                        theTarget = e;
                    }
                }
            }
        }

        if(theTarget == null)
        {
            return false;
        }

        StringBuffer msg=new StringBuffer(theDoll.getDollName() + " : ");
			
        if(theTarget!=null)
        {
            msg.append(theTarget.getEntityString());
            msg.append("(");
            msg.append(theTarget.entityId);
            msg.append(") ");
				
            theDoll.chatMessage(msg.toString(),1);
        }
        else
        {
            msg.append("No target");
            theDoll.chatMessage(msg.toString(),2);
        }
        
        return true;
        
    }

    @Override
    public boolean continueExecuting()
    {
        if(!theDoll.isEnable())
        {
            return false;
        }
        
        if(!theDoll.isPatrolMode())
        {
            return false;
        }

        if(theTarget.isDead)
        {
            return false;
        }

        if(theDoll.getNavigator().noPath())
        {
            return false;
        }
        return true;
    }

    @Override
    public void startExecuting()
    {
        fuse = 0;
    }

    @Override
    public void resetTask()
    {
    }

    @Override
    public void updateTask()
    {
        theDoll.getLookHelper().setLookPositionWithEntity(theTarget, 10F, theDoll.getVerticalFaceSpeed());
        if(!theDoll.getNavigator().tryMoveToEntityLiving(theTarget, speed))
        {
            return;
        }

        if(theDoll.getDistanceToEntity(theTarget) > 5f)
        {
            fuse = (fuse>0 ? fuse-1 : 0);
        }
        else
        {
            ++fuse;
            if(fuse==1)
            {
                theWorld.playSoundAtEntity(theDoll, "random.fuse", 1.0F, 0.5F);
            }
            if(fuse>30)
            {
                theWorld.createExplosion(theDoll, theDoll.posX, theDoll.posY, theDoll.posZ, explodeStrength,
                                         theWorld.getGameRules().getGameRuleBooleanValue("mobGriefing"));
                theDoll.onDeath(DamageSource.explosion);
                theDoll.setDead();
                fuse = 0;
            }
        }
    }
}
