package net.minecraft.src;

import java.util.*;
import java.util.regex.*;

public class EntityDollAIMineBlock extends EntityAIBase
{
    private EntityDoll theDoll;
    private World theWorld;
    private float speed;
    private boolean isTarget, isMining;
    private int targetX, mineX;
    private int targetY, mineY;
    private int targetZ, mineZ;
    private int counter;
    private int blockStrength;

    public static String targetBlock;
    public static int mineRange;
    public static double mineSpeed;
    
    public EntityDollAIMineBlock(EntityDoll entitydoll, float s)
    {
        theDoll = entitydoll;
        theWorld = entitydoll.worldObj;
        speed = s;
        counter = 0;
        setMutexBits(0x3);
    }

    @Override
    public boolean shouldExecute()
    {
        if(!theDoll.isEnable())
        {
            return false;
        }

        if(counter < 20)
        {
            ++counter;
            return false;
        }
        counter = 0;

        int dollposX = MathHelper.floor_double(theDoll.posX);
        int dollposY = MathHelper.floor_double(theDoll.posY + (double)theDoll.getEyeHeight());
        int dollposZ = MathHelper.floor_double(theDoll.posZ);
        Pattern targetPattern = Pattern.compile(targetBlock);
        Matcher targetMatcher;
        TreeMap<String, Integer> blockCount = new TreeMap<String, Integer>();
        targetX = mineRange;
        targetY = mineRange;
        targetZ = mineRange;
        isTarget = false;

        for(int dx=-mineRange; dx<=mineRange; ++dx)
        {
            for(int dy=-mineRange; dy<=mineRange; ++dy)
            {
                for(int dz=-mineRange; dz<=mineRange; ++dz)
                {
                    int distanceToBlock = Math.abs(dx)+Math.abs(dy)+Math.abs(dz);
                    if(distanceToBlock > mineRange)
                    {
                        continue;
                    }
                    Block b = Block.blocksList[theWorld.getBlockId(dollposX+dx, dollposY+dy, dollposZ+dz)];
                    if(b==null)
                    {
                        continue;
                    }
                    String blockName = getBlockName(b);
                    
                    //̌@Ώۂǂ
                    targetMatcher = targetPattern.matcher(blockName);
                    if(targetMatcher.find())
                    {
                        if(blockCount.containsKey(blockName))
                        {
                            int c = blockCount.get(blockName).intValue() + 1;
                            blockCount.put(blockName, new Integer(c));
                        }
                        else
                        {
                            blockCount.put(blockName, new Integer(1));
                        }

                        if(Math.abs(targetX)+Math.abs(targetY)+Math.abs(targetZ) >distanceToBlock)
                        {
                            if(canDigBlock(dollposX+dx, dollposY+dy, dollposZ+dz))
                            {
                                targetX = dx;
                                targetY = dy;
                                targetZ = dz;
                                isTarget = true;
                            }
                        }
                    }
                }
            }
        }
        
        // o͕̍쐬
        StringBuffer msg = new StringBuffer(theDoll.getDollName() + " : ");

        if(blockCount.isEmpty())
        {
            msg.append("No target");
            theDoll.chatMessage(msg.toString(),2);
        }
        else
        {
            Iterator it = blockCount.keySet().iterator();
            while(it.hasNext())
            {
                String s = (String)it.next();
                int v = blockCount.get(s).intValue();
                msg.append(s);
                msg.append("[");
                msg.append(v);
                msg.append("] ");
            }
            theDoll.chatMessage(msg.toString(),1);
        }
        
        if(theDoll.isStandbyMode() || theDoll.isRideonMode())
        {
            return false;
        }
        
        if(isTarget)
        {
            //@ΏۃubŇ
            mineX = 0;
            mineY = 0;
            mineZ = 0;
            isMining = false;

            for(int r=1; r<=mineRange; ++r)
            {
                boolean isAirBlock = false;
                int ii, jj, kk;
                ii = jj = kk = 0;
                for(int k=-1; k<=1; ++k)
                {
                    for(int j=-1; j<=1; ++j)
                    {
                        for(int i=-1; i<=1; ++i)
                        {
                            //JgubNɗאڂubNΏۂƂ
                            if(Math.abs(i)+Math.abs(j)+Math.abs(k) != 1)
                            {
                                continue;
                            }
                            if(Math.abs(targetX-(mineX+ii))+Math.abs(targetY-(mineY+jj))+Math.abs(targetZ-(mineZ+kk))
                               > Math.abs(targetX-(mineX+i))+Math.abs(targetY-(mineY+j))+Math.abs(targetZ-(mineZ+k)))
                            {
                                if(canDigBlock(dollposX+mineX+i, dollposY+mineY+j, dollposZ+mineZ+k))
                                {
                                    //^[QbgɂƂ߂@\ubN
                                    ii=i;
                                    jj=j;
                                    kk=k;
                                    isMining = true;
                                }
                                else if(theWorld.isAirBlock(dollposX+mineX+i, dollposY+mineY+j, dollposZ+mineZ+k))
                                {
                                    ii=i;
                                    jj=j;
                                    kk=k;                                    
                                }
                            }
                        }
                    }
                }
                mineX += ii;
                mineY += jj;
                mineZ += kk;
                if(isMining)
                {
                    break;
                }
            }
        
            // @ubN̐ݒ
            Block b = Block.blocksList[theWorld.getBlockId(mineX+dollposX, mineY+dollposY, mineZ+dollposZ)];
            Block t = Block.blocksList[theWorld.getBlockId(targetX+dollposX, targetY+dollposY, targetZ+dollposZ)];
            blockStrength = b == null ? 0 : MathHelper.floor_double(20.0*b.getBlockHardness(theWorld, mineX+dollposX, mineY+dollposY, mineZ+dollposZ)/mineSpeed);
            blockStrength = blockStrength < 0 ? 0 : blockStrength;
        
            StringBuilder debugout = new StringBuilder(theDoll.getDollName() + " : ");
            debugout.append(String.format("Taregt:%s(%d,%d,%d) Mining:%s(%d,%d,%d)",
                                          getBlockName(t), targetX, targetY, targetZ,
                                          getBlockName(b), mineX, mineY, mineZ));
            theDoll.chatMessage(debugout.toString(),3);
        
            mineX += dollposX;
            mineY += dollposY;
            mineZ += dollposZ;

        }

        return isMining;
    }

    @Override
    public boolean continueExecuting()
    {
        if(!theDoll.isEnable())
        {
            return false;
        }

        if(theDoll.isStandbyMode() || theDoll.isRideonMode())
        {
            return false;
        }

        return isMining;
    }

    @Override
    public void startExecuting()
    {
        counter = 0;
    }

    @Override
    public void resetTask()
    {
    }

    @Override
    public void updateTask()
    {
        //@Ώۂ違߂ÂƂ
        theDoll.getLookHelper().setLookPosition(mineX+0.5, mineY+0.5, mineZ+0.5,
                                                20F, theDoll.getVerticalFaceSpeed());
        theDoll.getNavigator().tryMoveToXYZ(mineX+0.5f, mineY+0.5f, mineZ+0.5f, speed);

        if(blockStrength <= 0)
        {
            //ubN
            Block b = Block.blocksList[theWorld.getBlockId(mineX, mineY, mineZ)];
            if(b != null)
            {
                StepSound stepsound = b.stepSound;
                b.dropBlockAsItem(theWorld, mineX, mineY, mineZ, theWorld.getBlockMetadata(mineX, mineY, mineZ), 0);
                theWorld.setBlockWithNotify(mineX, mineY, mineZ, 0);
                theWorld.playSoundEffect(mineX+0.5f, mineY+0.5f, mineZ+0.5f, stepsound.getBreakSound(),
                                         stepsound.getVolume(), stepsound.getPitch());
            }
            
            isMining = false;
            counter = 20;
        }
        else if(blockStrength > 0 && blockStrength%4 == 0)
        {
            int bid = theWorld.getBlockId(mineX, mineY, mineZ);
            if(bid == 0)
            {
                isMining = false;
                return;
            }
            Block b = Block.blocksList[bid];
            if(b != null)
            {
                StepSound stepsound = b.stepSound;
                theWorld.playSoundEffect(mineX+0.5f, mineY+0.5f, mineZ+0.5f, stepsound.getBreakSound(),
                                         (stepsound.getVolume() + 1.0f) / 8f, stepsound.getPitch() * 0.5f);
            }
        }
        --blockStrength;
    }

    // ̌@\ǂ肷
	private boolean canDigBlock(int i, int j, int k)
	{
		if(theWorld.isAirBlock(i, j, k))
		{
			return false;
		}
		
		Block b = Block.blocksList[theWorld.getBlockId(i, j, k)];
		if(b!=null)
		{
			if(b.getBlockHardness(theWorld, i, j, k) < 0f || b.getBlockHardness(theWorld, i, j, k) > Block.obsidian.getBlockHardness(theWorld, 0, 0, 0))
			{
				return false;
			}
			if(theWorld.getBlockId(i, j + 1, k) == Block.sand.blockID || theWorld.getBlockId(i, j + 1, k) == Block.gravel.blockID)
			{
				return false;
			}
		}
		
		int u=-1;
		while(theWorld.isAirBlock(i, j+u, k))
		{
			if(j+u<=0)
			{
				return false;
			}
			if(theWorld.getBlockMaterial(i, j+u-1, k)==Material.lava)
			{
				return false;
			}
			--u;
		}
		
		for(int ii=-1;ii<=1;++ii)
		{
			for(int jj=-1;jj<=1;++jj)
			{
				for(int kk=-1;kk<=1;++kk)
				{
					if((ii>0?ii:-ii)+(jj>0?jj:-jj)+(kk>0?kk:-kk)<=1)
					{
						if(theWorld.getBlockMaterial(i+ii, j+jj, k+kk)==Material.lava)
						{
							return false;
						}
						if(theWorld.getBlockMaterial(i+ii, j+jj, k+kk)==Material.water)
						{
							return false;
						}
					}
				}
			}
		}
		
		return true;
	}

    private String getBlockName(Block b)
    {
        if(b == null)
        {
            return "";
        }
    
        String blockName = b.getBlockName();
        if(blockName == null)
        {
            blockName = String.format("Block%d", b.blockID);
        }
        else
        {
            int dot = blockName.indexOf(".");
            if(dot != -1)
            {
                blockName = blockName.substring(dot+1);
            }
        }
        return blockName;
    }
}

