package net.minecraft.src;

import java.util.*;
import java.util.regex.*;

public class EntityDollAISearchSpawner extends EntityAIBase
{
    private EntityDoll theDoll;
    private World theWorld;
    private int chunkX, chunkZ;
    private int baseY;
    private int counter;
    private TreeMap<Integer, String> spawnerMap;
    
    public EntityDollAISearchSpawner(EntityDoll entitydoll)
    {
        theDoll = entitydoll;
        theWorld = entitydoll.worldObj;
        counter = 0;
        setMutexBits(0x4);
    }

    @Override
    public boolean shouldExecute()
    {
        if(!theDoll.isEnable())
        {
            return false;
        }

        counter = 0;

        chunkX = MathHelper.floor_double(theDoll.posX)/16;
        baseY = MathHelper.floor_double(theDoll.posY + (double)theDoll.getEyeHeight());
        chunkZ = MathHelper.floor_double(theDoll.posZ)/16;

        spawnerMap = new TreeMap<Integer, String>();

        return true;
    }

    @Override
    public boolean continueExecuting()
    {
        if(!theDoll.isEnable())
        {
            return false;
        }

        return counter <= 64;
    }

    @Override
    public void updateTask()
    {
        theDoll.searchDev = (float)(counter)/64F;

        for(int dx = -16; dx <= 32; ++dx)
        {
            for(int dz = -16; dz <= 32; ++dz)
            {
                for(int dy = -1; dy <= 1; dy+=2)
                {
                    if(counter == 0 && dy == 1)
                    {
                        continue;
                    }
                    int bid = theWorld.getBlockId(chunkX*16+dx, baseY+counter*dy, chunkZ*16+dz);
                    if(bid == Block.mobSpawner.blockID)
                    {
                        TileEntityMobSpawner spawner = (TileEntityMobSpawner)(theWorld.getBlockTileEntity(chunkX*16+dx, baseY+counter*dy, chunkZ*16+dz));
                        
                        if(spawner != null)
                        {
                            String mobname = spawner.getMobID();
                            if(mobname == null)
                            {
                                mobname = "unknown";
                            }
                            else if(mobname == "")
                            {
                                mobname = "unknown";
                            }

                            if(spawnerMap.containsKey(counter*dy))
                            {
                                String s = spawnerMap.get(counter*dy);
                                spawnerMap.put(counter*dy, String.format("%s,%s", s, mobname));
                            }
                            else
                            {
                                spawnerMap.put(counter*dy, mobname);
                            }
                        }

                    }
                }
            }
        }
        
        if(counter == 64)
        {
            StringBuffer msg=new StringBuffer(theDoll.getDollName() + " : ");
            for(int i=64; i>0; --i)
            {
                if(i%8 == 0)
                {
                    msg.append("-");
                }
                if(spawnerMap.containsKey(i))
                {
                    msg.append("[");
                    msg.append(spawnerMap.get(i));
                    msg.append("]");
                }
            }
            if(spawnerMap.containsKey(0))
            {
                msg.append("<[");
                msg.append(spawnerMap.get(0));
                msg.append("]>");
            }
            else
            {
                msg.append(getStrongholdDirection());
            }
            for(int i=-1; i>=-64; --i)
            {
                if(i%8 == 0)
                {
                    msg.append("-");
                }
                if(spawnerMap.containsKey(i))
                {
                    msg.append("[");
                    msg.append(spawnerMap.get(i));
                    msg.append("]");
                }
            }
            
            theDoll.chatMessage(msg.toString(),2);
        }
        
        ++counter;
    }

    //Ղ̕pvZ
	public String getStrongholdDirection()
	{
		ChunkPosition chunkposition = theWorld.findClosestStructure("Stronghold", (int)theDoll.posX, (int)theDoll.posY, (int)theDoll.posZ);
		
		if(chunkposition == null)
		{
			return "<->";
		}
		
		double vecX = chunkposition.x - theDoll.posX;
		double vecY = chunkposition.y - theDoll.posY;
		double vecZ = chunkposition.z - theDoll.posZ;
		double dist_sq = vecX*vecX + vecZ*vecZ + 0.01D;
		
		if(dist_sq<16.0D)
		{
			return "<C>";
		}
		
		if(vecX>=vecZ && vecX>=-vecZ)
		{
			return "<E>";
		}
		if(-vecX>=vecZ && -vecX>=-vecZ)
		{
			return "<W>";
		}
		if(vecZ>=vecX && vecZ>=-vecX)
		{
			return "<S>";
		}
		if(-vecZ>=vecX && -vecZ>=-vecX)
		{
			return "<N>";
		}
		return "<->";
	}
}

