package net.minecraft.src;

import java.util.*;
import java.util.regex.*;

public class EntityDollAIStealItem extends EntityAIBase
{
    private EntityDoll theDoll;
    private World theWorld;
    private EntityLiving theTarget;
    private float speed;

    public EntityDollAIStealItem(EntityDoll entitydoll, float s)
    {
        theDoll = entitydoll;
        theWorld = entitydoll.worldObj;
        speed = s;
        setMutexBits(0x3);
    }

    @Override
    public boolean shouldExecute()
    {
        if(!theDoll.isEnable())
        {
            return false;
        }

        theTarget = theDoll.getAttackTarget();
        if(theTarget == null)
        {
            return false;
        }

        if(theTarget instanceof EntitySheep)
        {
            return true;
        }
        else if(theTarget instanceof EntityChicken)
        {
            return true;
        }
        else if(theTarget instanceof EntityMooshroom)
        {
            return true;
        }
        else if(theTarget instanceof EntityIronGolem)
        {
            return true;
        }
        
        return false;
        
    }

    @Override
    public void startExecuting()
    {
    }
    
    @Override
    public boolean continueExecuting()
    {
        return theDoll.isEnable() && shouldExecute() && !theDoll.getNavigator().noPath();
    }

    @Override
    public void resetTask()
    {
        theDoll.getNavigator().clearPathEntity();
    }

    @Override
    public void updateTask()
    {
        theDoll.getNavigator().tryMoveToEntityLiving(theTarget, speed);
        theDoll.getLookHelper().setLookPositionWithEntity(theTarget, 10F, theDoll.getVerticalFaceSpeed());
            

        if(theDoll.getDistanceSqToEntity(theTarget) < 9f && theDoll.getEntitySenses().canSee(theTarget))
        {
            if(theTarget instanceof EntitySheep)
            {
                EntitySheep sheep = (EntitySheep)theTarget;
                Random rand = sheep.getRNG();
                if(!sheep.getSheared() && !sheep.isChild() && theDoll.getRNG().nextFloat()<0.1f)
                {
                    if (!theWorld.isRemote)
                    {
                        sheep.setSheared(true);
                        int i = 1 + rand.nextInt(3);

                        for (int j = 0; j < i; j++)
                        {
                            EntityItem entityitem = sheep.entityDropItem(new ItemStack(Block.cloth.blockID, 1, sheep.getFleeceColor()), 1.0F);
                            entityitem.motionY += rand.nextFloat() * 0.05F;
                            entityitem.motionX += (rand.nextFloat() - rand.nextFloat()) * 0.1F;
                            entityitem.motionZ += (rand.nextFloat() - rand.nextFloat()) * 0.1F;
                        }
                        theWorld.playSoundAtEntity(theTarget, "mob.sheep.shear", 0.5F, (theDoll.getRNG().nextFloat() - theDoll.getRNG().nextFloat()) * 0.2F + 1.0F);
                        theDoll.chatMessage(theDoll.getDollName() + " : Wool!",2);
                        theDoll.swingItem();
                        theTarget = null;
                    }
                }
            }
            else if(theTarget instanceof EntityChicken)
            {
                Random rand = theTarget.getRNG();
                if(theDoll.getRNG().nextFloat()<0.003f)
                {
                    theTarget.entityDropItem(new ItemStack(Item.feather,1), 0.0F);
                    theWorld.playSoundAtEntity(theTarget, "mob.sheep.shear", 0.5F, (rand.nextFloat() - rand.nextFloat()) * 0.2F + 1.0F);
                    theDoll.chatMessage(theDoll.getDollName() + " : Feather!",2);
                    theDoll.swingItem();
                    theTarget.attackEntityFrom(DamageSource.causeMobDamage(theDoll), 0);
                    theTarget = null;
                }
            }
            else if(theTarget instanceof EntityMooshroom)
            {
                Random rand = theTarget.getRNG();
                if(theDoll.getRNG().nextFloat()<0.006f)
                {
                    theTarget.entityDropItem(new ItemStack(Block.mushroomRed,1), 0.0F);
                    theWorld.playSoundAtEntity(theTarget, "mob.chickenplop", 1.0F, (rand.nextFloat() - rand.nextFloat()) * 0.2F + 1.0F);
                    theDoll.chatMessage(theDoll.getDollName() + " : Mushroom!",2);
                    theDoll.swingItem();
                    theTarget.attackEntityFrom(DamageSource.causeMobDamage(theDoll), 0);
                    theTarget = null;
                }
            }
            else if(theTarget instanceof EntityIronGolem)
            {
                EntityIronGolem golem = (EntityIronGolem)theTarget;
                Random rand = theTarget.getRNG();

                if(golem.getHoldRoseTick() == 0)
                {
                    if(theDoll.getRNG().nextFloat()<0.001f)
                    {
                        golem.setHoldingRose(true);
                    }
                }
                if(golem.getHoldRoseTick() != 0)
                {
                    golem.getLookHelper().setLookPositionWithEntity(theDoll, 30F, 30F);
                    if(golem.getEntitySenses().canSee(theDoll) && theDoll.getRNG().nextFloat()<0.01f)
                    {
                        golem.setHoldingRose(false);
                        theTarget.entityDropItem(new ItemStack(Block.plantRed,1), 0.0F);
                        theWorld.playSoundAtEntity(theTarget, "mob.chickenplop", 1.0F, (rand.nextFloat() - rand.nextFloat()) * 0.2F + 1.0F);
                        theDoll.chatMessage(theDoll.getDollName() + " : Flower!",2);
                        theTarget = null;
                    }
                }
            }
        }
    }
}
