package net.minecraft.src;

import java.util.*;
import java.util.regex.*;

public class EntityDollAITorcher extends EntityAIBase
{
    private EntityDoll theDoll;
    private World theWorld;
    
    public EntityDollAITorcher(EntityDoll entitydoll)
    {
        theDoll = entitydoll;
        theWorld = entitydoll.worldObj;
        setMutexBits(0x0);
    }

    @Override
    public boolean shouldExecute()
    {
        if(!theDoll.isEnable())
        {
            return false;
        }

        if(!isTorcher())
        {
            return false;
        }

        int dollposX = MathHelper.floor_double(theDoll.posX);
        int dollposY = MathHelper.floor_double(theDoll.posY + (double)theDoll.getEyeHeight());
        int dollposZ = MathHelper.floor_double(theDoll.posZ);

        int targetX = dollposX;
        int targetY = dollposY;
        int targetZ = dollposZ;

        while(!theWorld.isAirBlock(targetX, targetY, targetZ))
        {
            ++targetY;
        }
        while(theWorld.isAirBlock(targetX, targetY, targetZ))
        {
            --targetY;
        }
        ++targetY;

        if(targetY<dollposY-3 || targetY >dollposY+3)
        {
            return false;
        }

        if(!isValidLightLevel(targetX, targetY, targetZ))
        {
            return false;
        }
        
        int bid = theWorld.getBlockId(targetX, targetY-1, targetZ);
        Block b = Block.blocksList[bid];
        if(b != null && b.isOpaqueCube())
        {
            theDoll.decrStackSize(0, 1);
            theWorld.setBlockAndMetadataWithNotify(targetX, targetY, targetZ, Block.torchWood.blockID, 5);
        }

        return false;
    }

    @Override
    public boolean continueExecuting()
    {
        return false;
    }

    // g[`[ǂ
    public boolean isTorcher()
	{
		if(theDoll.getStackInSlot(0) == null)
		{
			return false;
		}
		return (theDoll.getStackInSlot(0).itemID == Block.torchWood.blockID);
	}

    // 邳珼u肷
    protected boolean isValidLightLevel(int i, int j, int k)
	{
		if (theWorld.getSavedLightValue(EnumSkyBlock.Sky, i, j, k) > theDoll.getRNG().nextInt(32))
		{
			return false;
		}
		int l = theWorld.getBlockLightValue(i, j, k);
		if (theWorld.isThundering())
		{
			int i1 = theWorld.skylightSubtracted;
			theWorld.skylightSubtracted = 10;
			l = theWorld.getBlockLightValue(i, j, k);
			theWorld.skylightSubtracted = i1;
		}
		return l <= 7;
	}
}

